' ========================================================================================
' GdipCloneCustomLineCap
' ========================================================================================

' SED_PBWIN
#COMPILE EXE
#DIM ALL
#INCLUDE "GDIPLUS.INC"

' ========================================================================================
' The following example creates a CustomLineCap with a stroke path, creates a second
' CustomLineCap by cloning the first, and then assigns the cloned cap to a Pen. It then
' draws a line by using the Pen object.
' ========================================================================================
SUB GDIP_CloneCustomLineCap (BYVAL hdc AS DWORD)

   LOCAL hStatus AS LONG
   LOCAL pGraphics AS DWORD
   LOCAL pCapPath AS DWORD
   LOCAL pFirstCap AS DWORD
   LOCAL pSecondCap AS DWORD
   LOCAL pPen AS DWORD
   DIM   PointsF(2) AS POINTF

   hStatus = GdipCreateFromHDC(hdc, pGraphics)

   ' // Create a path and add two lines to it.
   PointsF(0).x = -15 : PointsF(0).y = -15
   PointsF(1).x = 0 : PointsF(1).y = 0
   PointsF(2).x = 15 : PointsF(2).y = -15
   hStatus = GdipCreatePath(%FillModeAlternate, pCapPath)
   hStatus = GdipAddPathLine2(pCapPath, PointsF(0), 3)

   ' // Create a CustomLineCap
   hStatus = GdipCreateCustomLineCap(%NULL, pCapPath, %LineCapFlat, 0, pFirstCap)

   ' // Create a copy of FirstCap
   hStatus = GdipCloneCustomLineCap(pFirstCap, pSecondCap)

   ' // Create a Pen, assignd SecondCap as the custom end cap, and draw the line
   hStatus = GdipCreatePen1(GDIP_ARGB(255, 0, 0, 0), 1.0!, %UnitWorld, pPen)
   hStatus = GdipSetPenCustomEndCap(pPen, pSecondCap)

   ' // Draw a line using arrowPen.
   hStatus = GdipDrawLineI(pGraphics, pPen, 0, 0, 100, 100)

   ' // Cleanup
   IF pSecondCap THEN GdipDeleteCustomLineCap(pSecondCap)
   IF pFirstCap THEN GdipDeleteCustomLineCap(pFirstCap)
   IF pPen THEN GdipDeletePen(pPen)
   IF pCapPath THEN GdipDeletePath(pCapPath)
   IF pGraphics THEN GdipDeleteGraphics(pGraphics)

END SUB
' ========================================================================================

' ========================================================================================
' Main
' ========================================================================================
FUNCTION WINMAIN (BYVAL hInstance AS DWORD, BYVAL hPrevInstance AS DWORD, BYVAL lpszCmdLine AS ASCIIZ PTR, BYVAL nCmdShow AS LONG) AS LONG

   LOCAL hr AS LONG
   LOCAL hDlg AS DWORD
   LOCAL token AS DWORD
   LOCAL StartupInput AS GdiplusStartupInput

   ' Initialize GDI+
   StartupInput.GdiplusVersion = 1
   hr = GdiplusStartup(token, StartupInput, BYVAL %NULL)
   IF hr THEN
      MSGBOX "Error initializing GDI+"
      EXIT FUNCTION
   END IF

   DIALOG NEW 0, "GdipCloneCustomLineCap", , , 270, 170, %WS_OVERLAPPED OR %WS_THICKFRAME OR %WS_SYSMENU OR _
   %WS_MINIMIZEBOX OR %WS_MAXIMIZEBOX OR %WS_VISIBLE OR %DS_CENTER TO hDlg
   DIALOG SET COLOR hDlg, %BLACK, %WHITE
   DIALOG SHOW MODAL hDlg, CALL DlgProc

   ' Shutdown GDI+
   GdiplusShutdown token

END FUNCTION
' ========================================================================================

' ========================================================================================
' Main Dialog procedure
' ========================================================================================
CALLBACK FUNCTION DlgProc() AS LONG

   LOCAL hDC AS DWORD
   LOCAL ps AS PAINTSTRUCT

   SELECT CASE CBMSG

      CASE %WM_COMMAND
         SELECT CASE CBCTL
            CASE %IDCANCEL
               IF CBCTLMSG = %BN_CLICKED THEN DIALOG END CBHNDL, 0
         END SELECT

      CASE %WM_PAINT
         hDC = BeginPaint(CBHNDL, ps)
         GDIP_CloneCustomLineCap hDC
         EndPaint(CBHNDL, ps)

   END SELECT

END FUNCTION
' ========================================================================================
